# -*- coding: utf-8 -*-

"""
our app setting
"""

import re

from django.conf import settings

from fleetpings.utils import clean_setting

from packaging import version

# set default panels if none are set in local.py
AA_FLEETPINGS_USE_SLACK = clean_setting("AA_FLEETPINGS_USE_SLACK", False)
AA_FLEETPINGS_USE_DOCTRINES_FROM_FITTINGS_MODULE = clean_setting(
    "AA_FLEETPINGS_USE_DOCTRINES_FROM_FITTINGS_MODULE", False
)

# AA-GDPR
AVOID_CDN = clean_setting("AVOID_CDN", False)


def get_site_url():  # regex sso url
    """
    get the site url
    :return: string
    """

    regex = r"^(.+)\/s.+"
    matches = re.finditer(regex, settings.ESI_SSO_CALLBACK_URL, re.MULTILINE)
    url = "http://"

    for match in matches:
        url = match.groups()[0]  # first match

    return url


def timezones_installed() -> bool:
    """
    check if aa-timezones is installed
    :return: bool
    """

    return "timezones" in settings.INSTALLED_APPS


def optimer_installed() -> bool:
    """
    check if optimer_installed is installed
    :return: bool
    """

    return "allianceauth.optimer" in settings.INSTALLED_APPS


def get_timzones_version():
    """
    get the version of aa-timezones, when installed
    :return: string or None
    """

    if timezones_installed():
        from timezones import __version__ as timezones_version

        return timezones_version

    return None


def use_new_timezone_links() -> bool:
    """
    determins whether to use then new link format from aa-timezones or not
    the new link format has been introduced with aa-timezones v1.2.1
    :return: bool
    """

    return_value = True

    if get_timzones_version() and version.parse(get_timzones_version()) < version.parse(
        "1.2.1"
    ):
        return_value = False

    return return_value


def fittings_installed() -> bool:
    """
    check if fittings is installed
    :return: bool
    """

    return "fittings" in settings.INSTALLED_APPS


def avoid_cdn() -> bool:
    """
    check if we should aviod CDN usage
    :return: bool
    """

    return AVOID_CDN


def discord_service_installed() -> bool:
    """
    check if the Discord service is installed
    :return: bool
    """

    return "allianceauth.services.modules.discord" in settings.INSTALLED_APPS



# Generated by Django 3.1.1 on 2020-10-03 12:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("fleetpings", "0006_fleetdoctrine_restricted_to_group"),
    ]

    operations = [
        migrations.AlterField(
            model_name="discordpingtargets",
            name="notes",
            field=models.TextField(
                blank=True,
                help_text="You can add notes about this configuration here if you want",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="fleetcomm",
            name="notes",
            field=models.TextField(
                blank=True,
                help_text="You can add notes about this configuration here if you want",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="fleetdoctrine",
            name="notes",
            field=models.TextField(
                blank=True,
                help_text="You can add notes about this configuration here if you want",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="fleettype",
            name="notes",
            field=models.TextField(
                blank=True,
                help_text="You can add notes about this configuration here if you want",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="formuplocation",
            name="notes",
            field=models.TextField(
                blank=True,
                help_text="You can add notes about this configuration here if you want",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="notes",
            field=models.TextField(
                blank=True,
                help_text="You can add notes about this webhook here if you want",
                null=True,
            ),
        ),
    ]



# -*- coding: utf-8 -*-

"""
app config
"""

from django.apps import AppConfig

from fleetpings import __version__


class AaFleetpingsConfig(AppConfig):
    """
    application config
    """

    name = "fleetpings"
    label = "fleetpings"
    verbose_name = "Fleet Pings v{}".format(__version__)



# -*- coding: utf-8 -*-

"""
pages url config
"""

from django.conf.urls import url

from fleetpings import views


app_name: str = "fleetpings"

urlpatterns = [
    url(r"^$", views.index, name="index"),
    url(
        r"^create_optimer/$",
        views.create_optimer_on_preping,
        name="create_optimer",
    ),
]



# coding=utf-8

"""
form declarations
"""

from django.forms import ModelForm, TextInput

from fleetpings.models import FleetType


class FleetTypeAdminForm(ModelForm):
    """
    form definitions for the FleetType form in admin
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta
        """

        model = FleetType
        fields = "__all__"
        widgets = {
            "embed_color": TextInput(attrs={"type": "color"}),
        }



# -*- coding: utf-8 -*-

"""
the views
"""

from django.contrib.auth.decorators import login_required, permission_required
from django.db.models import Q
from django.http import JsonResponse
from django.shortcuts import render
from django.utils import timezone

from fleetpings import __title__
from fleetpings.app_settings import (
    AA_FLEETPINGS_USE_SLACK,
    AA_FLEETPINGS_USE_DOCTRINES_FROM_FITTINGS_MODULE,
    get_site_url,
    timezones_installed,
    optimer_installed,
    use_new_timezone_links,
    fittings_installed,
    avoid_cdn,
)
from fleetpings.models import (
    FleetComm,
    DiscordPingTargets,
    FleetType,
    Webhook,
    FleetDoctrine,
    FormupLocation,
)

if (
    fittings_installed() is True
    and AA_FLEETPINGS_USE_DOCTRINES_FROM_FITTINGS_MODULE is True
):
    from fittings.views import _get_docs_qs

if optimer_installed():
    from allianceauth.optimer.models import OpTimer


@login_required
@permission_required("fleetpings.basic_access")
def index(request):
    """
    Index view
    """
    fleet_comms = FleetComm.objects.filter(is_enabled=True).order_by("name")

    # which platform for pings we are using?
    platform_used = "Discord"
    if AA_FLEETPINGS_USE_SLACK is True:
        platform_used = "Slack"

    # do we use the doctrines from the fittings module, or our own defined?
    use_fleet_doctrines = False
    if (
        fittings_installed() is True
        and AA_FLEETPINGS_USE_DOCTRINES_FROM_FITTINGS_MODULE is True
    ):
        use_fleet_doctrines = True

    # get the webhooks for the used platform
    webhooks = (
        Webhook.objects.filter(
            Q(restricted_to_group__in=request.user.groups.all())
            | Q(restricted_to_group__isnull=True),
            type=platform_used,
            is_enabled=True,
        )
        .distinct()
        .order_by("name")
    )

    # get additional ping targets for discord
    additional_discord_ping_targets = {}
    if AA_FLEETPINGS_USE_SLACK is False:
        additional_discord_ping_targets = (
            DiscordPingTargets.objects.filter(
                Q(restricted_to_group__in=request.user.groups.all())
                | Q(restricted_to_group__isnull=True),
                is_enabled=True,
            )
            .distinct()
            .order_by("name")
        )

    # get fleet types
    fleet_types = (
        FleetType.objects.filter(
            Q(restricted_to_group__in=request.user.groups.all())
            | Q(restricted_to_group__isnull=True),
            is_enabled=True,
        )
        .distinct()
        .order_by("name")
    )

    # get doctrines
    if use_fleet_doctrines is True:
        groups = request.user.groups.all()
        doctrines = _get_docs_qs(request, groups).order_by("name")
    else:
        doctrines = (
            FleetDoctrine.objects.filter(
                Q(restricted_to_group__in=request.user.groups.all())
                | Q(restricted_to_group__isnull=True),
                is_enabled=True,
            )
            .distinct()
            .order_by("name")
        )

    # get formup locations
    formup_locations = FormupLocation.objects.filter(is_enabled=True).order_by("name")

    context = {
        "title": __title__,
        "additionalPingTargets": additional_discord_ping_targets,
        "additionalFleetTypes": fleet_types,
        "additionalPingWebhooks": webhooks,
        "fleetComms": fleet_comms,
        "fleetDoctrines": doctrines,
        "fleetFormupLocations": formup_locations,
        "site_url": get_site_url(),
        "timezones_installed": timezones_installed(),
        "optimer_installed": optimer_installed(),
        "use_new_timezone_links": use_new_timezone_links(),
        "fittings_installed": fittings_installed(),
        "mainCharacter": request.user.profile.main_character,
        "platformUsed": platform_used,
        "useFleetDoctrines": use_fleet_doctrines,
        "avoidCdn": avoid_cdn(),
    }

    return render(request, "fleetpings/index.html", context)


@login_required
@permission_required("fleetpings.basic_access")
def create_optimer_on_preping(request):
    """
    adding the planned fleet to the optimers
    :param request:
    :return:
    """

    post_time = timezone.now()
    character = request.user.profile.main_character

    optimer = OpTimer()
    optimer.doctrine = request.POST["fleet_doctrine"]
    optimer.system = request.POST["formup_location"]
    optimer.start = request.POST["formup_time"]
    optimer.duration = "-"
    optimer.operation_name = request.POST["fleet_name"]
    optimer.fc = request.POST["fleet_commander"]
    optimer.post_time = post_time
    optimer.eve_character = character
    optimer.save()

    return JsonResponse([True], safe=False)
